# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Version information for Invenio-Config-TUW.

This file is imported by ``invenio_config_tuw.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "2021.1"
